@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -77,6 +77,39 @@ function RBlockListQt(parent, addListener, showHeader) {
 
 RBlockListQt.prototype = new RTreeWidget();
 
+RBlockListQt.getBlockList = function() {
+    var appWin = EAction.getMainWindow();
+    return appWin.findChild("BlockList");
+};
+
+/**
+ * Add an additional action to be used in the context menu.
+ * Can be used by plugins to hook into the block list context menu.
+ */
+RBlockListQt.addContextMenuScriptFile = function(scriptFile) {
+    var blockList = RBlockListQt.getBlockList();
+    if (isNull(blockList)) {
+        return;
+    }
+
+    var scriptFiles = RBlockListQt.getContextMenuScriptFiles();
+    scriptFiles.push(scriptFile);
+    blockList.setProperty("ContextMenuScriptFiles", scriptFiles);
+};
+
+RBlockListQt.getContextMenuScriptFiles = function() {
+    var blockList = RBlockListQt.getBlockList();
+    if (isNull(blockList)) {
+        return [];
+    }
+
+    var scriptFiles = blockList.property("ContextMenuScriptFiles");
+    if (isNull(scriptFiles)) {
+        scriptFiles = [];
+    }
+    return scriptFiles;
+};
+
 RBlockListQt.prototype.contextMenuEvent = function(e) {
     var item = this.itemAt(e.pos());
     if (!isNull(item)) {
@@ -84,6 +117,7 @@ RBlockListQt.prototype.contextMenuEvent = function(e) {
     }
 
     var menu = new QMenu();
+
     RGuiAction.getByScriptFile("scripts/Block/ToggleBlockVisibility/ToggleBlockVisibility.js").addToMenu(menu);
     RGuiAction.getByScriptFile("scripts/Block/ShowAllBlocks/ShowAllBlocks.js").addToMenu(menu);
     RGuiAction.getByScriptFile("scripts/Block/HideAllBlocks/HideAllBlocks.js").addToMenu(menu);
@@ -95,6 +129,16 @@ RBlockListQt.prototype.contextMenuEvent = function(e) {
     RGuiAction.getByScriptFile("scripts/Block/SelectBlockReferences/SelectBlockReferences.js").addToMenu(menu);
     RGuiAction.getByScriptFile("scripts/Block/DeselectBlockReferences/DeselectBlockReferences.js").addToMenu(menu);
 
+    // add addtional context menu actions provided by plugins:
+    var scriptFiles = RBlockListQt.getContextMenuScriptFiles();
+    for (var i=0; i<scriptFiles.length; i++) {
+        var scriptFile = scriptFiles[i];
+        var a = RGuiAction.getByScriptFile(scriptFile);
+        if (!isNull(a)) {
+            a.addToMenu(menu);
+        }
+    }
+
     menu.exec(QCursor.pos());
 
     e.ignore();
@@ -228,7 +272,7 @@ RBlockListQt.prototype.updateBlocks = function(documentInterface) {
             continue;
         }
 
-        var item = this.getBlockItem(block);
+        var item = this.createBlockItem(block);
         this.addTopLevelItem(item);
         if (blockName===selectedBlockName) {
             selectedItem = item;
@@ -259,7 +303,7 @@ RBlockListQt.getBlockTitle = function(block) {
     return title;
 };
 
-RBlockListQt.prototype.getBlockItem = function(block) {
+RBlockListQt.prototype.createBlockItem = function(block) {
     var item = new QTreeWidgetItem();
     var name = block.getName();
 
@@ -370,6 +414,19 @@ RBlockListQt.prototype.getBlockName = function(item) {
     return item.data(BlockList.colName, Qt.UserRole);
 };
 
+/**
+ * \return The list item for the block with the given block name.
+ */
+RBlockListQt.getItem = function(widget, blockName) {
+    for (var i=0; i<widget.topLevelItemCount; i++) {
+        var item = widget.topLevelItem(i);
+        if (item.data(BlockList.colName, Qt.UserRole)===blockName) {
+            return item;
+        }
+    }
+    return undefined;
+};
+
 /**
  * Enable / disable buttons that edit, remove or insert the active block.
  */
